package com.example.ueberwachungssystem;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.pm.PackageManager;
import android.media.MediaPlayer;
import android.view.Surface;
import android.widget.Toast;
import android.widget.VideoView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.camera.core.CameraSelector;
import androidx.camera.core.Preview;
import androidx.camera.core.VideoCapture;
import androidx.camera.lifecycle.ProcessCameraProvider;
import androidx.camera.view.PreviewView;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.lifecycle.LifecycleOwner;

import com.google.common.util.concurrent.ListenableFuture;

import java.io.File;
import java.util.concurrent.ExecutionException;

public class DetectionRecorder {

    private final Context context;
    private ProcessCameraProvider cameraProvider;
    private VideoCapture videoCapture = null;
    private PreviewView previewView = null;
    private boolean isRunning = false;
    private boolean isVideoPlaying = false;


    // Constructor
    public DetectionRecorder(Context context) {
        this.context = context;
    }


    public void start() {
        if (isRunning)
            return;
        isRunning = true;

        // Request Camera Provider
        final ListenableFuture<ProcessCameraProvider> cameraProviderFuture = ProcessCameraProvider.getInstance(context);
        //Check for Camera availability
        cameraProviderFuture.addListener(() -> {
            try {
                cameraProvider = cameraProviderFuture.get();
                startCameraX(cameraProvider);
            } catch (ExecutionException | InterruptedException e) {
                // No errors need to be handled for this Future. This should never be reached.
            }
        }, ContextCompat.getMainExecutor(context));
    }


    @SuppressLint("RestrictedApi")
    public void stop() {
        if (!isRunning)
            return;

        videoCapture.stopRecording();
        cameraProvider.unbind(videoCapture);
        isRunning = false;
    }

    @SuppressLint("RestrictedApi")
    private void startCameraX(ProcessCameraProvider cameraProvider) {
        //cameraProvider.unbindAll();

        CameraSelector cameraSelector = new CameraSelector.Builder()
                .requireLensFacing(CameraSelector.LENS_FACING_BACK)
                .build();

        // Preview Use Case
        Preview preview = new Preview.Builder().build();
        if (previewView != null)
            preview.setSurfaceProvider(previewView.getSurfaceProvider());

        // Video Capture Use Case
        videoCapture = new VideoCapture.Builder()
                .setVideoFrameRate(30)
                .setTargetRotation(Surface.ROTATION_0)
                .build();

        cameraProvider.bindToLifecycle((LifecycleOwner) context, cameraSelector, preview, videoCapture);


        File dir = context.getFilesDir();
        String filename = "recording";
        String vidFilePath = dir.getAbsolutePath() + "/" + filename + ".mp4";
        File vidFile = new File(vidFilePath);

        if (ActivityCompat.checkSelfPermission(context, android.Manifest.permission.RECORD_AUDIO) != PackageManager.PERMISSION_GRANTED) {
            // TODO: Consider calling
            //    ActivityCompat#requestPermissions
            // here to request the missing permissions, and then overriding
            //   public void onRequestPermissionsResult(int requestCode, String[] permissions,
            //                                          int[] grantResults)
            // to handle the case where the user grants the permission. See the documentation
            // for ActivityCompat#requestPermissions for more details.
            return;
        }
        videoCapture.startRecording(
                new VideoCapture.OutputFileOptions.Builder(vidFile).build(),
                context.getMainExecutor(),
                new VideoCapture.OnVideoSavedCallback() {
                    @Override
                    public void onVideoSaved(@NonNull VideoCapture.OutputFileResults outputFileResults) {
                        Toast.makeText(context, "recording saved", Toast.LENGTH_SHORT).show();
                    }
                    @Override
                    public void onError(int videoCaptureError, @NonNull String message, @Nullable Throwable cause) {
                        Toast.makeText(context, "recording failed", Toast.LENGTH_SHORT).show();
                    }
                }
        );
    }

    /** Set PreviewView to show Image */
    public void setPreviewView(PreviewView previewView) {
        this.previewView = previewView;
    }

    private void playAudio() {
        MediaPlayer mp = new MediaPlayer();
        try {
            mp.setDataSource(context.getFilesDir() + "/audio.gpp");
            mp.prepare();
            mp.start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void playVideo(VideoView videoView) {
        if (isVideoPlaying)
            return;
        videoView.setVideoPath(context.getFilesDir() + "/recording.mp4");
        videoView.start();
    }
}
