package com.example.ueberwachungssystem;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.camera.view.PreviewView;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.camera.core.ExperimentalGetImage;

import android.Manifest;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;
import android.widget.VideoView;

import com.example.ueberwachungssystem.Detection.DetectionReport;
import com.example.ueberwachungssystem.Detection.Detector;
import com.example.ueberwachungssystem.Detection.VideoDetector;

import java.io.File;
import java.util.Arrays;

@ExperimentalGetImage
public class MainActivity extends AppCompatActivity {

    private static final int PERMISSION_REQUEST_CODE = 111;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        TextView textView = findViewById(R.id.textView);
        VideoView videoView = findViewById(R.id.videoView);
        PreviewView previewView = findViewById(R.id.previewView);

        File directory = getFilesDir();


        AudioRecorder ar = new AudioRecorder(this);


        //DetectionRecorder detectionRecorder = new DetectionRecorder(this);
        //detectionRecorder.setPreviewView(previewView);

        ToggleButton toggleButton = findViewById(R.id.previewButton);
        ToggleButton toggleButton2 = findViewById(R.id.toggleButton2);

        toggleButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                getRecordVideoAccess();
                File[] files = directory.listFiles();
                textView.setText(Arrays.toString(files));

                if (isRecordVideoAllowed() && toggleButton.isChecked())
                {
                    ar.startRecording();
                    //vd.startRecording();
                    //vd.playVideo(videoView);
                    //detectionRecorder.playVideo(videoView);
                    //detectionRecorder.start();
                }
                else {
                    ar.stopRecording();
                    //vd.stopRecording();

                    //detectionRecorder.stop();
                }
            }
        });

        toggleButton2.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (toggleButton.isChecked())
                {
                    ar.playAudio();
                }
                else {
                    ar.deleteRecording();
                }
            }
        });
    }

    private boolean isRecordVideoAllowed() {
        return ContextCompat.checkSelfPermission(this, Manifest.permission.CAMERA) == PackageManager.PERMISSION_GRANTED &&
                ContextCompat.checkSelfPermission(this, Manifest.permission.RECORD_AUDIO) == PackageManager.PERMISSION_GRANTED;
    }

    private void getRecordVideoAccess() {
        if (!isRecordVideoAllowed()) {
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.CAMERA, Manifest.permission.RECORD_AUDIO}, PERMISSION_REQUEST_CODE);
        }
    }


    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        if (requestCode == PERMISSION_REQUEST_CODE && grantResults.length > 0) {

            boolean cameraRights = grantResults[0] == PackageManager.PERMISSION_GRANTED;
            boolean recordAudioRights = grantResults[1] == PackageManager.PERMISSION_GRANTED;

            if (cameraRights && recordAudioRights) {
                Toast.makeText(this, "permissions granted", Toast.LENGTH_LONG).show();
            } else {
                Toast.makeText(this, "permissions denied", Toast.LENGTH_LONG).show();
            }
        }
    }
}